/*
 * Generated by gdbus-codegen 2.33.4. DO NOT EDIT.
 *
 * The license of this code is the same as for the source it was derived from.
 */

#ifndef __UM_REALM_GENERATED_H__
#define __UM_REALM_GENERATED_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.realmd.Provider */

#define UM_REALM_TYPE_PROVIDER (um_realm_provider_get_type ())
#define UM_REALM_PROVIDER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), UM_REALM_TYPE_PROVIDER, UmRealmProvider))
#define UM_REALM_IS_PROVIDER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), UM_REALM_TYPE_PROVIDER))
#define UM_REALM_PROVIDER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), UM_REALM_TYPE_PROVIDER, UmRealmProviderIface))

struct _UmRealmProvider;
typedef struct _UmRealmProvider UmRealmProvider;
typedef struct _UmRealmProviderIface UmRealmProviderIface;

struct _UmRealmProviderIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_discover) (
    UmRealmProvider *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_string,
    const gchar *arg_operation_id);

  const gchar * (*get_name) (UmRealmProvider *object);

  GVariant * (*get_realms) (UmRealmProvider *object);

  const gchar * (*get_version) (UmRealmProvider *object);

};

GType um_realm_provider_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *um_realm_provider_interface_info (void);
guint um_realm_provider_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void um_realm_provider_complete_discover (
    UmRealmProvider *object,
    GDBusMethodInvocation *invocation,
    gint relevance,
    GVariant *realm);



/* D-Bus method calls: */
void um_realm_provider_call_discover (
    UmRealmProvider *proxy,
    const gchar *arg_string,
    const gchar *arg_operation_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean um_realm_provider_call_discover_finish (
    UmRealmProvider *proxy,
    gint *out_relevance,
    GVariant **out_realm,
    GAsyncResult *res,
    GError **error);

gboolean um_realm_provider_call_discover_sync (
    UmRealmProvider *proxy,
    const gchar *arg_string,
    const gchar *arg_operation_id,
    gint *out_relevance,
    GVariant **out_realm,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
const gchar *um_realm_provider_get_name (UmRealmProvider *object);
gchar *um_realm_provider_dup_name (UmRealmProvider *object);
void um_realm_provider_set_name (UmRealmProvider *object, const gchar *value);

const gchar *um_realm_provider_get_version (UmRealmProvider *object);
gchar *um_realm_provider_dup_version (UmRealmProvider *object);
void um_realm_provider_set_version (UmRealmProvider *object, const gchar *value);

GVariant *um_realm_provider_get_realms (UmRealmProvider *object);
GVariant *um_realm_provider_dup_realms (UmRealmProvider *object);
void um_realm_provider_set_realms (UmRealmProvider *object, GVariant *value);


/* ---- */

#define UM_REALM_TYPE_PROVIDER_PROXY (um_realm_provider_proxy_get_type ())
#define UM_REALM_PROVIDER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), UM_REALM_TYPE_PROVIDER_PROXY, UmRealmProviderProxy))
#define UM_REALM_PROVIDER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), UM_REALM_TYPE_PROVIDER_PROXY, UmRealmProviderProxyClass))
#define UM_REALM_PROVIDER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), UM_REALM_TYPE_PROVIDER_PROXY, UmRealmProviderProxyClass))
#define UM_REALM_IS_PROVIDER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), UM_REALM_TYPE_PROVIDER_PROXY))
#define UM_REALM_IS_PROVIDER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), UM_REALM_TYPE_PROVIDER_PROXY))

typedef struct _UmRealmProviderProxy UmRealmProviderProxy;
typedef struct _UmRealmProviderProxyClass UmRealmProviderProxyClass;
typedef struct _UmRealmProviderProxyPrivate UmRealmProviderProxyPrivate;

struct _UmRealmProviderProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  UmRealmProviderProxyPrivate *priv;
};

struct _UmRealmProviderProxyClass
{
  GDBusProxyClass parent_class;
};

GType um_realm_provider_proxy_get_type (void) G_GNUC_CONST;

void um_realm_provider_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
UmRealmProvider *um_realm_provider_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
UmRealmProvider *um_realm_provider_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void um_realm_provider_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
UmRealmProvider *um_realm_provider_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
UmRealmProvider *um_realm_provider_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define UM_REALM_TYPE_PROVIDER_SKELETON (um_realm_provider_skeleton_get_type ())
#define UM_REALM_PROVIDER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), UM_REALM_TYPE_PROVIDER_SKELETON, UmRealmProviderSkeleton))
#define UM_REALM_PROVIDER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), UM_REALM_TYPE_PROVIDER_SKELETON, UmRealmProviderSkeletonClass))
#define UM_REALM_PROVIDER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), UM_REALM_TYPE_PROVIDER_SKELETON, UmRealmProviderSkeletonClass))
#define UM_REALM_IS_PROVIDER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), UM_REALM_TYPE_PROVIDER_SKELETON))
#define UM_REALM_IS_PROVIDER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), UM_REALM_TYPE_PROVIDER_SKELETON))

typedef struct _UmRealmProviderSkeleton UmRealmProviderSkeleton;
typedef struct _UmRealmProviderSkeletonClass UmRealmProviderSkeletonClass;
typedef struct _UmRealmProviderSkeletonPrivate UmRealmProviderSkeletonPrivate;

struct _UmRealmProviderSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  UmRealmProviderSkeletonPrivate *priv;
};

struct _UmRealmProviderSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType um_realm_provider_skeleton_get_type (void) G_GNUC_CONST;

UmRealmProvider *um_realm_provider_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.realmd.Diagnostics */

#define UM_REALM_TYPE_DIAGNOSTICS (um_realm_diagnostics_get_type ())
#define UM_REALM_DIAGNOSTICS(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), UM_REALM_TYPE_DIAGNOSTICS, UmRealmDiagnostics))
#define UM_REALM_IS_DIAGNOSTICS(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), UM_REALM_TYPE_DIAGNOSTICS))
#define UM_REALM_DIAGNOSTICS_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), UM_REALM_TYPE_DIAGNOSTICS, UmRealmDiagnosticsIface))

struct _UmRealmDiagnostics;
typedef struct _UmRealmDiagnostics UmRealmDiagnostics;
typedef struct _UmRealmDiagnosticsIface UmRealmDiagnosticsIface;

struct _UmRealmDiagnosticsIface
{
  GTypeInterface parent_iface;

  void (*diagnostics) (
    UmRealmDiagnostics *object,
    const gchar *arg_data,
    const gchar *arg_operation_id);

};

GType um_realm_diagnostics_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *um_realm_diagnostics_interface_info (void);
guint um_realm_diagnostics_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus signal emissions functions: */
void um_realm_diagnostics_emit_diagnostics (
    UmRealmDiagnostics *object,
    const gchar *arg_data,
    const gchar *arg_operation_id);



/* ---- */

#define UM_REALM_TYPE_DIAGNOSTICS_PROXY (um_realm_diagnostics_proxy_get_type ())
#define UM_REALM_DIAGNOSTICS_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), UM_REALM_TYPE_DIAGNOSTICS_PROXY, UmRealmDiagnosticsProxy))
#define UM_REALM_DIAGNOSTICS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), UM_REALM_TYPE_DIAGNOSTICS_PROXY, UmRealmDiagnosticsProxyClass))
#define UM_REALM_DIAGNOSTICS_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), UM_REALM_TYPE_DIAGNOSTICS_PROXY, UmRealmDiagnosticsProxyClass))
#define UM_REALM_IS_DIAGNOSTICS_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), UM_REALM_TYPE_DIAGNOSTICS_PROXY))
#define UM_REALM_IS_DIAGNOSTICS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), UM_REALM_TYPE_DIAGNOSTICS_PROXY))

typedef struct _UmRealmDiagnosticsProxy UmRealmDiagnosticsProxy;
typedef struct _UmRealmDiagnosticsProxyClass UmRealmDiagnosticsProxyClass;
typedef struct _UmRealmDiagnosticsProxyPrivate UmRealmDiagnosticsProxyPrivate;

struct _UmRealmDiagnosticsProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  UmRealmDiagnosticsProxyPrivate *priv;
};

struct _UmRealmDiagnosticsProxyClass
{
  GDBusProxyClass parent_class;
};

GType um_realm_diagnostics_proxy_get_type (void) G_GNUC_CONST;

void um_realm_diagnostics_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
UmRealmDiagnostics *um_realm_diagnostics_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
UmRealmDiagnostics *um_realm_diagnostics_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void um_realm_diagnostics_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
UmRealmDiagnostics *um_realm_diagnostics_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
UmRealmDiagnostics *um_realm_diagnostics_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define UM_REALM_TYPE_DIAGNOSTICS_SKELETON (um_realm_diagnostics_skeleton_get_type ())
#define UM_REALM_DIAGNOSTICS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), UM_REALM_TYPE_DIAGNOSTICS_SKELETON, UmRealmDiagnosticsSkeleton))
#define UM_REALM_DIAGNOSTICS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), UM_REALM_TYPE_DIAGNOSTICS_SKELETON, UmRealmDiagnosticsSkeletonClass))
#define UM_REALM_DIAGNOSTICS_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), UM_REALM_TYPE_DIAGNOSTICS_SKELETON, UmRealmDiagnosticsSkeletonClass))
#define UM_REALM_IS_DIAGNOSTICS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), UM_REALM_TYPE_DIAGNOSTICS_SKELETON))
#define UM_REALM_IS_DIAGNOSTICS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), UM_REALM_TYPE_DIAGNOSTICS_SKELETON))

typedef struct _UmRealmDiagnosticsSkeleton UmRealmDiagnosticsSkeleton;
typedef struct _UmRealmDiagnosticsSkeletonClass UmRealmDiagnosticsSkeletonClass;
typedef struct _UmRealmDiagnosticsSkeletonPrivate UmRealmDiagnosticsSkeletonPrivate;

struct _UmRealmDiagnosticsSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  UmRealmDiagnosticsSkeletonPrivate *priv;
};

struct _UmRealmDiagnosticsSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType um_realm_diagnostics_skeleton_get_type (void) G_GNUC_CONST;

UmRealmDiagnostics *um_realm_diagnostics_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.realmd.Kerberos */

#define UM_REALM_TYPE_KERBEROS (um_realm_kerberos_get_type ())
#define UM_REALM_KERBEROS(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), UM_REALM_TYPE_KERBEROS, UmRealmKerberos))
#define UM_REALM_IS_KERBEROS(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), UM_REALM_TYPE_KERBEROS))
#define UM_REALM_KERBEROS_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), UM_REALM_TYPE_KERBEROS, UmRealmKerberosIface))

struct _UmRealmKerberos;
typedef struct _UmRealmKerberos UmRealmKerberos;
typedef struct _UmRealmKerberosIface UmRealmKerberosIface;

struct _UmRealmKerberosIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_change_permitted_logins) (
    UmRealmKerberos *object,
    GDBusMethodInvocation *invocation,
    const gchar *const *arg_add,
    const gchar *const *arg_remove,
    const gchar *arg_operation_id);

  gboolean (*handle_enroll_with_credential_cache) (
    UmRealmKerberos *object,
    GDBusMethodInvocation *invocation,
    GVariant *arg_kerberos_cache,
    GVariant *arg_options,
    const gchar *arg_operation_id);

  gboolean (*handle_enroll_with_password) (
    UmRealmKerberos *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_principal,
    const gchar *arg_password,
    GVariant *arg_options,
    const gchar *arg_operation_id);

  gboolean (*handle_unenroll_with_credential_cache) (
    UmRealmKerberos *object,
    GDBusMethodInvocation *invocation,
    GVariant *arg_kerberos_cache,
    GVariant *arg_options,
    const gchar *arg_operation_id);

  gboolean (*handle_unenroll_with_password) (
    UmRealmKerberos *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_principal,
    const gchar *arg_password,
    GVariant *arg_options,
    const gchar *arg_operation_id);

  GVariant * (*get_details) (UmRealmKerberos *object);

  const gchar * (*get_domain) (UmRealmKerberos *object);

  gboolean  (*get_enrolled) (UmRealmKerberos *object);

  const gchar * (*get_login_format) (UmRealmKerberos *object);

  const gchar * (*get_name) (UmRealmKerberos *object);

  const gchar *const * (*get_permitted_logins) (UmRealmKerberos *object);

  const gchar * (*get_suggested_administrator) (UmRealmKerberos *object);

};

GType um_realm_kerberos_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *um_realm_kerberos_interface_info (void);
guint um_realm_kerberos_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void um_realm_kerberos_complete_enroll_with_password (
    UmRealmKerberos *object,
    GDBusMethodInvocation *invocation);

void um_realm_kerberos_complete_enroll_with_credential_cache (
    UmRealmKerberos *object,
    GDBusMethodInvocation *invocation);

void um_realm_kerberos_complete_unenroll_with_password (
    UmRealmKerberos *object,
    GDBusMethodInvocation *invocation);

void um_realm_kerberos_complete_unenroll_with_credential_cache (
    UmRealmKerberos *object,
    GDBusMethodInvocation *invocation);

void um_realm_kerberos_complete_change_permitted_logins (
    UmRealmKerberos *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void um_realm_kerberos_call_enroll_with_password (
    UmRealmKerberos *proxy,
    const gchar *arg_principal,
    const gchar *arg_password,
    GVariant *arg_options,
    const gchar *arg_operation_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean um_realm_kerberos_call_enroll_with_password_finish (
    UmRealmKerberos *proxy,
    GAsyncResult *res,
    GError **error);

gboolean um_realm_kerberos_call_enroll_with_password_sync (
    UmRealmKerberos *proxy,
    const gchar *arg_principal,
    const gchar *arg_password,
    GVariant *arg_options,
    const gchar *arg_operation_id,
    GCancellable *cancellable,
    GError **error);

void um_realm_kerberos_call_enroll_with_credential_cache (
    UmRealmKerberos *proxy,
    GVariant *arg_kerberos_cache,
    GVariant *arg_options,
    const gchar *arg_operation_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean um_realm_kerberos_call_enroll_with_credential_cache_finish (
    UmRealmKerberos *proxy,
    GAsyncResult *res,
    GError **error);

gboolean um_realm_kerberos_call_enroll_with_credential_cache_sync (
    UmRealmKerberos *proxy,
    GVariant *arg_kerberos_cache,
    GVariant *arg_options,
    const gchar *arg_operation_id,
    GCancellable *cancellable,
    GError **error);

void um_realm_kerberos_call_unenroll_with_password (
    UmRealmKerberos *proxy,
    const gchar *arg_principal,
    const gchar *arg_password,
    GVariant *arg_options,
    const gchar *arg_operation_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean um_realm_kerberos_call_unenroll_with_password_finish (
    UmRealmKerberos *proxy,
    GAsyncResult *res,
    GError **error);

gboolean um_realm_kerberos_call_unenroll_with_password_sync (
    UmRealmKerberos *proxy,
    const gchar *arg_principal,
    const gchar *arg_password,
    GVariant *arg_options,
    const gchar *arg_operation_id,
    GCancellable *cancellable,
    GError **error);

void um_realm_kerberos_call_unenroll_with_credential_cache (
    UmRealmKerberos *proxy,
    GVariant *arg_kerberos_cache,
    GVariant *arg_options,
    const gchar *arg_operation_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean um_realm_kerberos_call_unenroll_with_credential_cache_finish (
    UmRealmKerberos *proxy,
    GAsyncResult *res,
    GError **error);

gboolean um_realm_kerberos_call_unenroll_with_credential_cache_sync (
    UmRealmKerberos *proxy,
    GVariant *arg_kerberos_cache,
    GVariant *arg_options,
    const gchar *arg_operation_id,
    GCancellable *cancellable,
    GError **error);

void um_realm_kerberos_call_change_permitted_logins (
    UmRealmKerberos *proxy,
    const gchar *const *arg_add,
    const gchar *const *arg_remove,
    const gchar *arg_operation_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean um_realm_kerberos_call_change_permitted_logins_finish (
    UmRealmKerberos *proxy,
    GAsyncResult *res,
    GError **error);

gboolean um_realm_kerberos_call_change_permitted_logins_sync (
    UmRealmKerberos *proxy,
    const gchar *const *arg_add,
    const gchar *const *arg_remove,
    const gchar *arg_operation_id,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
const gchar *um_realm_kerberos_get_name (UmRealmKerberos *object);
gchar *um_realm_kerberos_dup_name (UmRealmKerberos *object);
void um_realm_kerberos_set_name (UmRealmKerberos *object, const gchar *value);

const gchar *um_realm_kerberos_get_domain (UmRealmKerberos *object);
gchar *um_realm_kerberos_dup_domain (UmRealmKerberos *object);
void um_realm_kerberos_set_domain (UmRealmKerberos *object, const gchar *value);

GVariant *um_realm_kerberos_get_details (UmRealmKerberos *object);
GVariant *um_realm_kerberos_dup_details (UmRealmKerberos *object);
void um_realm_kerberos_set_details (UmRealmKerberos *object, GVariant *value);

const gchar *um_realm_kerberos_get_suggested_administrator (UmRealmKerberos *object);
gchar *um_realm_kerberos_dup_suggested_administrator (UmRealmKerberos *object);
void um_realm_kerberos_set_suggested_administrator (UmRealmKerberos *object, const gchar *value);

gboolean um_realm_kerberos_get_enrolled (UmRealmKerberos *object);
void um_realm_kerberos_set_enrolled (UmRealmKerberos *object, gboolean value);

const gchar *um_realm_kerberos_get_login_format (UmRealmKerberos *object);
gchar *um_realm_kerberos_dup_login_format (UmRealmKerberos *object);
void um_realm_kerberos_set_login_format (UmRealmKerberos *object, const gchar *value);

const gchar *const *um_realm_kerberos_get_permitted_logins (UmRealmKerberos *object);
gchar **um_realm_kerberos_dup_permitted_logins (UmRealmKerberos *object);
void um_realm_kerberos_set_permitted_logins (UmRealmKerberos *object, const gchar *const *value);


/* ---- */

#define UM_REALM_TYPE_KERBEROS_PROXY (um_realm_kerberos_proxy_get_type ())
#define UM_REALM_KERBEROS_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), UM_REALM_TYPE_KERBEROS_PROXY, UmRealmKerberosProxy))
#define UM_REALM_KERBEROS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), UM_REALM_TYPE_KERBEROS_PROXY, UmRealmKerberosProxyClass))
#define UM_REALM_KERBEROS_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), UM_REALM_TYPE_KERBEROS_PROXY, UmRealmKerberosProxyClass))
#define UM_REALM_IS_KERBEROS_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), UM_REALM_TYPE_KERBEROS_PROXY))
#define UM_REALM_IS_KERBEROS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), UM_REALM_TYPE_KERBEROS_PROXY))

typedef struct _UmRealmKerberosProxy UmRealmKerberosProxy;
typedef struct _UmRealmKerberosProxyClass UmRealmKerberosProxyClass;
typedef struct _UmRealmKerberosProxyPrivate UmRealmKerberosProxyPrivate;

struct _UmRealmKerberosProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  UmRealmKerberosProxyPrivate *priv;
};

struct _UmRealmKerberosProxyClass
{
  GDBusProxyClass parent_class;
};

GType um_realm_kerberos_proxy_get_type (void) G_GNUC_CONST;

void um_realm_kerberos_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
UmRealmKerberos *um_realm_kerberos_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
UmRealmKerberos *um_realm_kerberos_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void um_realm_kerberos_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
UmRealmKerberos *um_realm_kerberos_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
UmRealmKerberos *um_realm_kerberos_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define UM_REALM_TYPE_KERBEROS_SKELETON (um_realm_kerberos_skeleton_get_type ())
#define UM_REALM_KERBEROS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), UM_REALM_TYPE_KERBEROS_SKELETON, UmRealmKerberosSkeleton))
#define UM_REALM_KERBEROS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), UM_REALM_TYPE_KERBEROS_SKELETON, UmRealmKerberosSkeletonClass))
#define UM_REALM_KERBEROS_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), UM_REALM_TYPE_KERBEROS_SKELETON, UmRealmKerberosSkeletonClass))
#define UM_REALM_IS_KERBEROS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), UM_REALM_TYPE_KERBEROS_SKELETON))
#define UM_REALM_IS_KERBEROS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), UM_REALM_TYPE_KERBEROS_SKELETON))

typedef struct _UmRealmKerberosSkeleton UmRealmKerberosSkeleton;
typedef struct _UmRealmKerberosSkeletonClass UmRealmKerberosSkeletonClass;
typedef struct _UmRealmKerberosSkeletonPrivate UmRealmKerberosSkeletonPrivate;

struct _UmRealmKerberosSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  UmRealmKerberosSkeletonPrivate *priv;
};

struct _UmRealmKerberosSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType um_realm_kerberos_skeleton_get_type (void) G_GNUC_CONST;

UmRealmKerberos *um_realm_kerberos_skeleton_new (void);


G_END_DECLS

#endif /* __UM_REALM_GENERATED_H__ */
